# $Id: Message.pm,v 1.5 2004/04/11 23:36:20 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
# All Rights Reserved. See file COPYRIGHT for details.
# 
# This module is part of Event::RPC, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Event::RPC::Message;

use Carp;
use strict;
use Storable;

sub get_sock			{ shift->{sock}				}

sub get_buffer			{ shift->{buffer}			}
sub get_length			{ shift->{length}			}
sub get_written			{ shift->{written}			}

sub set_buffer			{ shift->{buffer}		= $_[1]	}
sub set_length			{ shift->{length}		= $_[1]	}
sub set_written			{ shift->{written}		= $_[1]	}

sub new {
	my $class = shift;
	my %par = @_;
	my ($sock) = $par{'sock'};
	
	$sock->blocking(1);
	
	my $self = bless {
		sock	=> $sock,
		buffer	=> undef,
		length	=> 0,
		written => 0,
	}, $class;

	return $self;
}

sub read {
	my $self = shift;

	if ( not defined $self->{buffer} ) {
		my $length_packed;
		my $rc = sysread ($self->get_sock, $length_packed, 4);
		die "Can't read message header" if not defined $rc;
		die "DISCONNECTED" if $rc == 0;
		$self->{length} = unpack("N", $length_packed);
		die "Incoming message too big"
			if $self->{length} > 4194304;
	}

	my $buffer_length = length($self->{buffer});

	my $rc = sysread (
		$self->get_sock,
		$self->{buffer},
		$self->{length} - $buffer_length,
		$buffer_length
	);

	return if not defined $rc;
	die "DISCONNECTED" if $rc == 0;

	$buffer_length = length($self->{buffer});

	return if $self->{length} != $buffer_length;

	my $data = Storable::thaw($self->{buffer});

	$self->{buffer} = undef;
	$self->{length} = 0;

	return $data;
}

sub read_blocked {
	my $self = shift;
	
	my $rc;
	$rc = $self->read while not defined $rc;
	
	return $rc;
}

sub write {
	my $self = shift;
	my ($data) = @_;

	if ( not defined $self->{buffer} ) {
		my $packed = Storable::freeze ($data);
		$self->{buffer} = pack("N", length($packed)).$packed;
		$self->{length} = length($self->{buffer});
		$self->{written} = 0;
	}

	my $rc = syswrite (
		$self->get_sock,
		$self->{buffer},
		$self->{length}-$self->{written},
		$self->{written},
	);

	return if not defined $rc;

	$self->{written} += $rc;
	
	if ( $self->{written} == $self->{length} ) {
		$self->{buffer} = undef;
		$self->{length} = 0;
		return 1;
	}
	return;
}

sub write_blocked {
	my $self = shift;
	my ($data) = @_;
	
	$self->write($data) and return;
	
	my $finished = 0;
	$finished = $self->write while not $finished;
	
	1;
}

1;

